/*
  Copyright Jesús Rubio <jesusprubio@gmail.com>

  This code may only be used under the MIT license found at
  https://opensource.org/licenses/MIT.
*/

'use strict';

// https://developer.shodan.io/api/exploits/rest

const utils = require('./utils');

const apiSection = 'exploits';
// Optional parameters for each function.
const optsExpected = {
  search: ['facets', 'page'],
  count: ['facets'],
};

async function req(endpoint, query, key, opts = {}) {
  const partialUrl = `/api/${endpoint}`;

  if (!query) {
    throw new Error(utils.createErrMandatory('query'));
  }

  let qs = { query, key };
  qs = utils.buildQs(qs, partialUrl, optsExpected[endpoint], opts);

  return utils.apiRequest(apiSection, partialUrl, qs, opts.timeout);
}

module.exports.search = async (query, key, opts = {}) =>
  req('search', query, key, opts);

module.exports.count = async (query, key, opts = {}) =>
  req('count', query, key, opts);
